@@ -13,8 +13,8 @@ from .quant_base import QuantBase
 
 
 class QuantRelu(QuantBase):
-    def __init__(self, name, module, bit_width, logger, writer):
-        super().__init__(name, module, bit_width, logger, writer)
+    def __init__(self, name, module, bit_width, writer):
+        super().__init__(name, module, bit_width, writer)
         self.activation_positive = True
         self.weight_fl = -1
 
@@ -23,8 +23,8 @@ class QuantRelu(QuantBase):
 
 
 class QuantLRelu(QuantBase):
-    def __init__(self, name, module, bit_width, logger, writer):
-        super().__init__(name, module, bit_width, logger, writer)
+    def __init__(self, name, module, bit_width, writer):
+        super().__init__(name, module, bit_width, writer)
         self.activation_positive = False
         self.weight_fl = -1
 
@@ -33,8 +33,8 @@ class QuantLRelu(QuantBase):
 
 
 class QuantPRelu(QuantBase):
-    def __init__(self, name, module, bit_width, logger, writer):
-        super().__init__(name, module, bit_width, logger, writer)
+    def __init__(self, name, module, bit_width, writer):
+        super().__init__(name, module, bit_width, writer)
         self.activation_positive = False
         self.weight_fl = -1
 
@@ -43,8 +43,8 @@ class QuantPRelu(QuantBase):
 
 
 class QuantLogSoftmax(QuantBase):
-    def __init__(self, name, module, bit_width, logger, writer):
-        super().__init__(name, module, bit_width, logger, writer)
+    def __init__(self, name, module, bit_width, writer):
+        super().__init__(name, module, bit_width, writer)
         self.activation_positive = False
         self.weight_fl = -1
 
