@@ -14,8 +14,8 @@ from .quant_base import *
 
 
 class QuantInput(QuantBase):
-    def __init__(self, name, module, bit_width, logger, writer):
-        super().__init__(name, module, bit_width, logger, writer)
+    def __init__(self, name, module, bit_width, writer):
+        super().__init__(name, module, bit_width, writer)
         self.weight_fl = -1
         self.activation_positive = False
 
@@ -24,8 +24,8 @@ class QuantInput(QuantBase):
 
 
 class QuantYOLO(QuantBase):
-    def __init__(self, name, module, bit_width, logger, writer):
-        super().__init__(name, module, bit_width, logger, writer)
+    def __init__(self, name, module, bit_width, writer):
+        super().__init__(name, module, bit_width, writer)
         self.weight_fl = -1
 
     def quantize_params(self):
