@@ -14,14 +14,14 @@ from .quant_base import QuantBase
 
 # support dilation convolution
 class QuantConv(QuantBase):
-    def __init__(self, name, module, bit_width, logger, writer):
-        super().__init__(name, module, bit_width, logger, writer)
+    def __init__(self, name, module, bit_width, writer):
+        super().__init__(name, module, bit_width, writer)
         self.activation_positive = False
 
 
 class QuantConvTranspose(QuantBase):
-    def __init__(self, name, module, bit_width, logger, writer):
-        super().__init__(name, module, bit_width, logger, writer)
+    def __init__(self, name, module, bit_width, writer):
+        super().__init__(name, module, bit_width, writer)
         self.activation_positive = False
 
 
