@@ -14,8 +14,8 @@ from ..qnq_ops import *
 
 
 class QuantTorchFuncGeneral(QuantBase):
-    def __init__(self, name, module, bit_width, logger, writer):
-        super().__init__(name, module, bit_width, logger, writer)
+    def __init__(self, name, module, bit_width, writer):
+        super().__init__(name, module, bit_width, writer)
         self.activation_positive = False
         self.weight_fl = -1
 
