@@ -13,8 +13,8 @@ from .quant_base import QuantBase
 
 
 class QuantUpsample(QuantBase):
-    def __init__(self, name, module, bit_width, logger, writer):
-        super().__init__(name, module, bit_width, logger, writer)
+    def __init__(self, name, module, bit_width, writer):
+        super().__init__(name, module, bit_width, writer)
         self.activation_positive = False
 
     def quantize_params(self):
@@ -22,8 +22,8 @@ class QuantUpsample(QuantBase):
 
 
 class QuantEmbedding(QuantBase):
-    def __init__(self, name, module, bit_width, logger, writer):
-        super().__init__(name, module, bit_width, logger, writer)
+    def __init__(self, name, module, bit_width, writer):
+        super().__init__(name, module, bit_width, writer)
         self.activation_positive = False
 
 
