@@ -8,7 +8,9 @@
 # Attention:
 #     1. Nothing
 
-import json
+import sys
+import yaml
+import logging
 from .operators import *
 from .utils import *
 
@@ -20,18 +22,22 @@ class QNQ():
                  model,
                  save_path="./checkpoints/test",
                  config_file=None,
-                 ext_metrics=None,
+                 metrics=None,
+                 metrics_light=None,
                  steper=None):
         # init parameters
         self.model = model
         self.save_path = save_path
         self.config_file = config_file
-        # ext_metrics return a float value for search
-        self.ext_metrics = ext_metrics
+        # metrics return a float value for search
+        self.metrics = metrics
+        # metrics_light return a float value for search, I suggest testset contain 1/10 of calibration datasets.
+        self.metrics_light = metrics_light
         self.steper = steper
 
-        # init abstract log path and logger
-        self.logger, self.log_path = lognow(self.save_path)
+        # init abstract log path, logger, writer
+        self.log_path = setup_logging(self.save_path)
+        self.logger = logging.getLogger("qnq")
         self.writer = SummaryWriter(log_dir=self.log_path)
 
         # load custom config
@@ -41,9 +47,9 @@ class QNQ():
             if not os.path.exists(os.path.dirname(self.config_file)):
                 os.makedirs(os.path.dirname(self.config_file))
             self.init_config()
-            self.logger.warning("Detected no config, creating...")
-            self.logger.warning("Please edit config and re-run the script.")
-            exit(0)
+            self.logger.error("Detected no config, creating...")
+            self.logger.error("Please edit config and re-run the script.")
+            sys.exit()
 
         # processing custom config
         # exec fuse
@@ -60,23 +66,27 @@ class QNQ():
             quant_layer = qnq_switch.get(type(module), None)
             if quant_layer:
                 # initialize quant_layer
-                quant_layer = quant_layer(name, module,
-                                          self.qconfig[name]['bit_width'],
-                                          self.logger, self.writer)
+                quant_layer = quant_layer(
+                    name, module, self.qparams["qlayers"][name]['bit_width'],
+                    self.writer)
 
                 #! 1. layer config (load)
                 #! load - save - default(quant_base)
                 #? more robust here
                 # do not load weight_fl, if origin is -1
                 if quant_layer.weight_fl != -1:
-                    quant_layer.weight_fl = self.qconfig[name]['weight_fl']
+                    quant_layer.weight_fl = self.qparams["qlayers"][name][
+                        'weight_fl']
                 # do not load activation_fl, if origin is -1
                 if quant_layer.activation_fl != -1:
-                    quant_layer.activation_fl = self.qconfig[name][
+                    quant_layer.activation_fl = self.qparams["qlayers"][name][
                         'activation_fl']
-                quant_layer.fuse_flag = self.qconfig[name]['fuse_flag']
-                quant_layer.group_flag = self.qconfig[name]['group_flag']
-                quant_layer.scale_flag = self.qconfig[name]['scale_flag']
+                quant_layer.fuse_flag = self.qparams["qlayers"][name][
+                    'fuse_flag']
+                quant_layer.group_flag = self.qparams["qlayers"][name][
+                    'group_flag']
+                quant_layer.scale_flag = self.qparams["qlayers"][name][
+                    'scale_flag']
 
                 # append quant_layer and name
                 self.quant_layer_list.append(quant_layer)
@@ -112,11 +122,26 @@ class QNQ():
                 else:
                     layer.trim_params(debug_bw=layer.bit_width,
                                       debug_fl=test_fl)
-                    metrics_test = self.ext_metrics()
-                    metrics.append(metrics_test)
-                    # info log
-                    self.logger.info("      weight_fl: " + str(test_fl) +
-                                     ", metrics: " + str(metrics_test))
+                    if self.metrics_light and self.metrics_light(
+                    ) > self.metrics_light_threshold:
+                        metrics_test = self.metrics()
+                        metrics.append(metrics_test)
+                        # info log
+                        self.logger.info("      weight_fl: " + str(test_fl) +
+                                         ", metrics: " + str(metrics_test))
+                    else:
+                        metrics_test = -1
+                        # info log
+                        self.logger.info("      weight_fl: " + str(test_fl) +
+                                         ", metrics: " + str(metrics_test) +
+                                         ", below metrics_light_threshold ")
+
+            # if need end search
+            if end_search(metrics):
+                self.logger.error(
+                    "Search Failed, metrics have been below or equals 0, please contact the developer."
+                )
+                sys.exit()
 
             # get maximum metrics' index
             layer.weight_fl = metrics.index(max(metrics))
@@ -144,11 +169,27 @@ class QNQ():
                 else:
                     layer.trim_activation(debug_bw=layer.bit_width,
                                           debug_fl=test_fl)
-                    metrics_test = self.ext_metrics()
-                    metrics.append(metrics_test)
-                    # info log
-                    self.logger.info("      activation_fl: " + str(test_fl) +
-                                     ", metrics: " + str(metrics_test))
+                    if self.metrics_light and self.metrics_light(
+                    ) > self.metrics_light_threshold:
+                        metrics_test = self.metrics()
+                        metrics.append(metrics_test)
+                        # info log
+                        self.logger.info("      weight_fl: " + str(test_fl) +
+                                         ", metrics: " + str(metrics_test))
+                    else:
+                        metrics_test = -1
+                        # info log
+                        self.logger.info("      weight_fl: " + str(test_fl) +
+                                         ", metrics: " + str(metrics_test) +
+                                         ", below metrics_light_threshold ")
+
+            # if need end search
+            if end_search(metrics):
+                self.logger.error(
+                    "Search Failed, metrics have been below or equals 0, please contact the developer."
+                )
+                sys.exit()
+
             # get maximum metrics' index
             layer.activation_fl = metrics.index(max(metrics))
 
@@ -170,7 +211,7 @@ class QNQ():
                 for layer in layers:
                     layer.trim_activation(debug_bw=layer.bit_width,
                                           debug_fl=test_fl)
-                metrics_test = self.ext_metrics()
+                metrics_test = self.metrics()
                 metrics.append(metrics_test)
                 # info log
                 self.logger.info("      activation_fl: " + str(test_fl) +
@@ -186,6 +227,13 @@ class QNQ():
 
             return max(metrics), metrics.index(max(metrics))
 
+        def end_search(metrics_list):
+            # todo, if metrics contain negative number??
+            if max(metrics_list) <= 0:
+                return True
+            else:
+                return False
+
         def do_quantize_params():
             # quantize params
             quantize_params()
@@ -193,7 +241,7 @@ class QNQ():
             self.logger.info("  Testing quantized params metrics...")
             for layer in self.quant_layer_list:
                 layer.trim_params()
-            metrics_params = self.ext_metrics()
+            metrics_params = self.metrics()
             self.logger.info("    Quantized params metrics:" +
                              str(metrics_params))
 
@@ -202,7 +250,7 @@ class QNQ():
         def do_search_params(metrics_mid, metrics_origin):
             self.logger.info("Searching params...")
             #? only search when metrics drop 2 * self.threshold
-            if metrics_origin - metrics_mid >= 2 * self.search_threshold:
+            if metrics_origin - metrics_mid >= 2 * self.search_params_threshold:
                 self.logger.info("  Need to search, starting...")
                 # recover un-trim before search
                 for layer in self.quant_layer_list:
@@ -216,9 +264,9 @@ class QNQ():
                                          " Checking " + layer.name +
                                          "'s weight_fl.")
                         layer.trim_params()
-                        metrics_tmp = self.ext_metrics()
+                        metrics_tmp = self.metrics()
                         #? only search when metrics drop >= self.threshold
-                        if metrics_mid - metrics_tmp >= self.search_threshold:
+                        if metrics_mid - metrics_tmp >= self.search_params_threshold:
                             self.logger.info("    Need to search, weight_fl=" +
                                              str(layer.weight_fl) +
                                              ", metrics=" + str(metrics_tmp))
@@ -251,7 +299,7 @@ class QNQ():
             self.logger.info("  Testing quantized activation metrics...")
             for layer in self.quant_layer_list:
                 layer.trim_activation()
-            metrics_activation = self.ext_metrics()
+            metrics_activation = self.metrics()
             self.logger.info("    Quantized activation metrics: " +
                              str(metrics_activation))
             return metrics_activation
@@ -259,7 +307,7 @@ class QNQ():
         def do_search_activation(metrics_mid, metrics_origin):
             self.logger.info("Searching activation...")
             #? only search only search when metrics drop 2 * self.threshold
-            if metrics_origin - metrics_mid >= 2 * self.search_threshold:
+            if metrics_origin - metrics_mid >= 2 * self.search_activation_threshold:
                 self.logger.info("  Need to search, starting...")
                 #todo valide
                 #? remove handler
@@ -276,10 +324,10 @@ class QNQ():
                                          " Checking " + layer.name +
                                          "'s activation_fl.")
                         layer.trim_activation()
-                        metrics_tmp = self.ext_metrics()
+                        metrics_tmp = self.metrics()
                         #? only search when metrics drop >= self.threshold
                         if abs(metrics_tmp -
-                               metrics_mid) >= self.search_threshold:
+                               metrics_mid) >= self.search_activation_threshold:
                             self.logger.info(
                                 "    Need to search, activation_fl=" +
                                 str(layer.activation_fl) + ", metrics=" +
@@ -327,8 +375,8 @@ class QNQ():
                                  str(metrics_cat))
             return metrics_cat
 
-        assert self.ext_metrics, "Search need a metrics."
-        #! 1. qconfig (show)
+        assert self.metrics, "Search need a metrics."
+        #! 1. qparams(show)
         #! show - save - load - default
         self.logger.info("Configuration:")
         self.logger.info("  Log_path: " + str(self.log_path))
@@ -340,9 +388,14 @@ class QNQ():
         self.logger.info("  Search_activation: " + str(self.search_activation))
         self.logger.info("  Search_group: " + str(self.search_group))
         self.logger.info("  Group_num: " + str(self.group_num))
-        self.logger.info("  Threshold: " + str(self.search_threshold))
+        self.logger.info("  Search_params_threshold: " + str(self.search_params_threshold))
+        self.logger.info("  Search_activation_threshold: " + str(self.search_activation_threshold))
+        self.logger.info("  Metrics_light: " +
+                         str(bool(self.metrics_light_threshold)))
+        self.logger.info("  Metrics_light_threshold: " +
+                         str(self.metrics_light_threshold))
         self.logger.info("  Testing origin metrics...")
-        metrics_final = metrics_origin = metrics_mid = self.ext_metrics()
+        metrics_final = metrics_origin = metrics_mid = self.metrics()
         self.logger.info("    Initialization metrics: " + str(metrics_mid))
 
         if self.quantize_params:
@@ -407,54 +460,64 @@ class QNQ():
         print(".", end=" ")
 
     def save_config(self, config_file=None):
-        #! 2. qconfig (save)
-        qconfig = {}
-        qconfig['config'] = {}
-        qconfig['config']['fuse_all'] = self.fuse_all
-        qconfig['config']['group_num'] = self.group_num
-        qconfig['config']['quantize_params'] = self.quantize_params
-        qconfig['config']['search_params'] = self.search_params
-        qconfig['config']['quantize_activation'] = self.quantize_activation
-        qconfig['config']['search_activation'] = self.search_activation
-        qconfig['config']['search_group'] = self.search_group
-        qconfig['config']['search_threshold'] = self.search_threshold
-        qconfig['config']['notes'] = "Value '-1' means no operation."
+        qparams = {}
+        #! 2. qparams(save)
+        qparams['qconfig'] = {}
+        qparams['qconfig']['fuse_all'] = self.fuse_all
+        qparams['qconfig']['group_num'] = self.group_num
+        qparams['qconfig']['quantize_params'] = self.quantize_params
+        qparams['qconfig']['search_params'] = self.search_params
+        qparams['qconfig']['quantize_activation'] = self.quantize_activation
+        qparams['qconfig']['search_activation'] = self.search_activation
+        qparams['qconfig']['search_group'] = self.search_group
+        qparams['qconfig']['search_params_threshold'] = self.search_params_threshold
+        qparams['qconfig']['search_activation_threshold'] = self.search_activation_threshold
+        qparams['qconfig'][
+            'metrics_light_threshold'] = self.metrics_light_threshold
+        qparams['qconfig']['notes'] = "Value '-1' means no operation."
 
         #! 2. layer config (save)
+        qparams['qlayers'] = {}
         for layer in self.quant_layer_list:
-            qconfig[layer.name] = {}
-            qconfig[layer.name]['bit_width'] = int(layer.bit_width)
-            qconfig[layer.name]['weight_fl'] = int(layer.weight_fl)
-            qconfig[layer.name]['activation_fl'] = int(layer.activation_fl)
-            qconfig[layer.name]['fuse_flag'] = int(layer.fuse_flag)
-            qconfig[layer.name]['group_flag'] = int(layer.group_flag)
-            qconfig[layer.name]['scale_flag'] = layer.scale_flag
-            qconfig[layer.name]['layer_type'] = str(type(layer.module))[8:-2]
+            qparams['qlayers'][layer.name] = {}
+            qparams['qlayers'][layer.name]['bit_width'] = int(layer.bit_width)
+            qparams['qlayers'][layer.name]['weight_fl'] = int(layer.weight_fl)
+            qparams['qlayers'][layer.name]['activation_fl'] = int(
+                layer.activation_fl)
+            qparams['qlayers'][layer.name]['fuse_flag'] = int(layer.fuse_flag)
+            qparams['qlayers'][layer.name]['group_flag'] = int(
+                layer.group_flag)
+            qparams['qlayers'][layer.name]['scale_flag'] = layer.scale_flag
+            qparams['qlayers'][layer.name]['layer_type'] = str(
+                type(layer.module))[8:-2]
 
         if config_file:
             with open(config_file, 'w+') as file:
-                json.dump(qconfig, file)
+                yaml.dump(qparams, file, sort_keys=False, indent=4)
         else:
-            with open(self.log_path + '/quant.json', 'w+') as file:
-                json.dump(qconfig, file)
+            with open(self.log_path + '/quant.yaml', 'w+') as file:
+                yaml.dump(qparams, file, sort_keys=False, indent=4)
 
     def load_config(self):
         # open config
         with open(self.config_file, 'r') as f:
-            self.qconfig = json.load(f)
-        #! 3. qconfig (load)
-        self.fuse_all = self.qconfig['config']['fuse_all']
-        self.group_num = self.qconfig['config']['group_num']
-        self.quantize_params = self.qconfig['config']['quantize_params']
-        self.search_params = self.qconfig['config']['search_params']
-        self.quantize_activation = self.qconfig['config'][
+            self.qparams = yaml.load(f, Loader=yaml.SafeLoader)
+        #! 3. qparams(load)
+        self.fuse_all = self.qparams['qconfig']['fuse_all']
+        self.group_num = self.qparams['qconfig']['group_num']
+        self.quantize_params = self.qparams['qconfig']['quantize_params']
+        self.search_params = self.qparams['qconfig']['search_params']
+        self.quantize_activation = self.qparams['qconfig'][
             'quantize_activation']
-        self.search_activation = self.qconfig['config']['search_activation']
-        self.search_group = self.qconfig['config']['search_group']
-        self.search_threshold = self.qconfig['config']['search_threshold']
+        self.search_activation = self.qparams['qconfig']['search_activation']
+        self.search_group = self.qparams['qconfig']['search_group']
+        self.search_params_threshold = self.qparams['qconfig']['search_params_threshold']
+        self.search_activation_threshold = self.qparams['qconfig']['search_activation_threshold']
+        self.metrics_light_threshold = self.qparams['qconfig'][
+            'metrics_light_threshold']
 
     def init_config(self):
-        #! 4. qconfig (default)
+        #! 4. qparams(default)
         self.fuse_all = True
         self.group_num = 0
         self.quantize_params = True
@@ -462,7 +525,9 @@ class QNQ():
         self.quantize_activation = True
         self.search_activation = True
         self.search_group = True
-        self.search_threshold = 0.002  # 0.2%
+        self.search_params_threshold = 0.002  # 0.2%
+        self.search_activation_threshold = 0.002  # 0.2%
+        self.metrics_light_threshold = 0.5  # assume accuracy was metrics
         self.bit_width = 8
 
         #! 3. layer config (default) (check quant_base)
@@ -473,7 +538,7 @@ class QNQ():
             quant_layer = qnq_switch.get(type(module), None)
             if quant_layer:
                 quant_layer = quant_layer(name, module, self.bit_width,
-                                          self.logger, self.writer)
+                                          self.writer)
                 self.quant_layer_list.append(quant_layer)
         # save init_config file
         self.save_config(self.config_file)
